# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.action_response import ActionResponse
from ..types.response_browser_info_result import ResponseBrowserInfoResult
from .raw_client import AsyncRawBrowserClient, RawBrowserClient
from .types.action import Action

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BrowserClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBrowserClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBrowserClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBrowserClient
        """
        return self._raw_client

    def get_browser_info(self, *, request_options: typing.Optional[RequestOptions] = None) -> ResponseBrowserInfoResult:
        """
        Get information about browser, like cdp url, viewport size, etc.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseBrowserInfoResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.browser.get_browser_info()
        """
        _response = self._raw_client.get_browser_info(request_options=request_options)
        return _response.data

    def take_screenshot(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.Iterator[bytes]:
        """
        Take a screenshot of the current display.

        Returns:
            StreamingResponse: PNG image data with proper headers

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            Screenshot image
        """
        with self._raw_client.take_screenshot(request_options=request_options) as r:
            yield from r.data

    def execute_action(
        self, *, request: Action, request_options: typing.Optional[RequestOptions] = None
    ) -> ActionResponse:
        """
        Execute a validated action on the current display.

        Parameters
        ----------
        request : Action

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionResponse
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox
        from agent_sandbox.browser import Action_MoveTo

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.browser.execute_action(
            request=Action_MoveTo(
                x=1.1,
                y=1.1,
            ),
        )
        """
        _response = self._raw_client.execute_action(request=request, request_options=request_options)
        return _response.data


class AsyncBrowserClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBrowserClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBrowserClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBrowserClient
        """
        return self._raw_client

    async def get_browser_info(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ResponseBrowserInfoResult:
        """
        Get information about browser, like cdp url, viewport size, etc.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseBrowserInfoResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.browser.get_browser_info()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_browser_info(request_options=request_options)
        return _response.data

    async def take_screenshot(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.AsyncIterator[bytes]:
        """
        Take a screenshot of the current display.

        Returns:
            StreamingResponse: PNG image data with proper headers

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            Screenshot image
        """
        async with self._raw_client.take_screenshot(request_options=request_options) as r:
            async for _chunk in r.data:
                yield _chunk

    async def execute_action(
        self, *, request: Action, request_options: typing.Optional[RequestOptions] = None
    ) -> ActionResponse:
        """
        Execute a validated action on the current display.

        Parameters
        ----------
        request : Action

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionResponse
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox
        from agent_sandbox.browser import Action_MoveTo

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.browser.execute_action(
                request=Action_MoveTo(
                    x=1.1,
                    y=1.1,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.execute_action(request=request, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.response import Response
from ..types.sandbox_response import SandboxResponse
from .raw_client import AsyncRawSandboxClient, RawSandboxClient


class SandboxClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSandboxClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSandboxClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSandboxClient
        """
        return self._raw_client

    def get_sandbox_context(self, *, request_options: typing.Optional[RequestOptions] = None) -> SandboxResponse:
        """
        Get sandbox environment information

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SandboxResponse
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.sandbox.get_sandbox_context()
        """
        _response = self._raw_client.get_sandbox_context(request_options=request_options)
        return _response.data

    def python_packages(self, *, request_options: typing.Optional[RequestOptions] = None) -> Response:
        """
        Get installed packages by language

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Response
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.sandbox.python_packages()
        """
        _response = self._raw_client.python_packages(request_options=request_options)
        return _response.data

    def nodejs_packages(self, *, request_options: typing.Optional[RequestOptions] = None) -> Response:
        """
        Get installed packages by language

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Response
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.sandbox.nodejs_packages()
        """
        _response = self._raw_client.nodejs_packages(request_options=request_options)
        return _response.data


class AsyncSandboxClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSandboxClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSandboxClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSandboxClient
        """
        return self._raw_client

    async def get_sandbox_context(self, *, request_options: typing.Optional[RequestOptions] = None) -> SandboxResponse:
        """
        Get sandbox environment information

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SandboxResponse
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.sandbox.get_sandbox_context()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_sandbox_context(request_options=request_options)
        return _response.data

    async def python_packages(self, *, request_options: typing.Optional[RequestOptions] = None) -> Response:
        """
        Get installed packages by language

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Response
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.sandbox.python_packages()


        asyncio.run(main())
        """
        _response = await self._raw_client.python_packages(request_options=request_options)
        return _response.data

    async def nodejs_packages(self, *, request_options: typing.Optional[RequestOptions] = None) -> Response:
        """
        Get installed packages by language

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Response
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.sandbox.nodejs_packages()


        asyncio.run(main())
        """
        _response = await self._raw_client.nodejs_packages(request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from abc import ABC, abstractmethod

from ..types.sandbox_response import SandboxResponse


class BaseProvider(ABC):
    """
    Base class for cloud provider implementations.
    
    All cloud providers should inherit from this class and implement
    the required sandbox management methods.
    """
    
    @abstractmethod
    def create_sandbox(self, function_id: str, **kwargs) -> SandboxResponse:
        """
        Create a new sandbox instance.
        
        Parameters
        ----------
        function_id : str
            The function ID for the sandbox
            
        Returns
        -------
        SandboxResponse
            The response containing sandbox creation details
        """
        pass
    
    @abstractmethod
    def delete_sandbox(self, function_id: str, sandbox_id: str, **kwargs) -> SandboxResponse:
        """
        Delete an existing sandbox instance.
        
        Parameters
        ----------
        function_id : str
            The function ID of the sandbox
        sandbox_id : str
            The ID of the sandbox to delete
            
        Returns
        -------
        SandboxResponse
            The response containing deletion status
        """
        pass
    
    @abstractmethod
    def get_sandbox(self, function_id: str, sandbox_id: str, **kwargs) -> SandboxResponse:
        """
        Get details of an existing sandbox instance.
        
        Parameters
        ----------
        function_id : str
            The function ID of the sandbox
        sandbox_id : str
            The ID of the sandbox to retrieve
            
        Returns
        -------
        SandboxResponse
            The response containing sandbox details
        """
        pass
    
    @abstractmethod
    def list_sandboxes(self, function_id: str, **kwargs) -> SandboxResponse:
        """
        List all sandbox instances for a function.
        
        Parameters
        ----------
        function_id : str
            The function ID to list sandboxes for
            
        Returns
        -------
        SandboxResponse
            The response containing list of sandboxes
        """
        pass



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from ..types.response_shell_command_result import ResponseShellCommandResult
from ..types.response_shell_kill_result import ResponseShellKillResult
from ..types.response_shell_view_result import ResponseShellViewResult
from ..types.response_shell_wait_result import ResponseShellWaitResult
from ..types.response_shell_write_result import ResponseShellWriteResult

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawShellClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def exec_command(
        self,
        *,
        command: str,
        id: typing.Optional[str] = OMIT,
        exec_dir: typing.Optional[str] = OMIT,
        async_mode: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ResponseShellCommandResult]:
        """
        Execute command in the specified shell session
        Supports SSE streaming if Accept header contains 'text/event-stream'

        Parameters
        ----------
        command : str
            Shell command to execute

        id : typing.Optional[str]
            Unique identifier of the target shell session, if not provided, one will be automatically created

        exec_dir : typing.Optional[str]
            Working directory for command execution (must use absolute path)

        async_mode : typing.Optional[bool]
            Whether to execute command asynchronously (default: False for async, False for synchronous execution)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ResponseShellCommandResult]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/shell/exec",
            method="POST",
            json={
                "id": id,
                "exec_dir": exec_dir,
                "command": command,
                "async_mode": async_mode,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseShellCommandResult,
                    parse_obj_as(
                        type_=ResponseShellCommandResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def view_shell(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ResponseShellViewResult]:
        """
        View output of the specified shell session
        Supports SSE streaming if Accept header contains 'text/event-stream'

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ResponseShellViewResult]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/shell/view",
            method="POST",
            json={
                "id": id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseShellViewResult,
                    parse_obj_as(
                        type_=ResponseShellViewResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def wait_for_process(
        self, *, id: str, seconds: typing.Optional[int] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ResponseShellWaitResult]:
        """
        Wait for the process in the specified shell session to return

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        seconds : typing.Optional[int]
            Wait time (seconds)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ResponseShellWaitResult]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/shell/wait",
            method="POST",
            json={
                "id": id,
                "seconds": seconds,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseShellWaitResult,
                    parse_obj_as(
                        type_=ResponseShellWaitResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def write_to_process(
        self, *, id: str, input: str, press_enter: bool, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ResponseShellWriteResult]:
        """
        Write input to the process in the specified shell session

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        input : str
            Input content to write to the process

        press_enter : bool
            Whether to press enter key after input

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ResponseShellWriteResult]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/shell/write",
            method="POST",
            json={
                "id": id,
                "input": input,
                "press_enter": press_enter,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseShellWriteResult,
                    parse_obj_as(
                        type_=ResponseShellWriteResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def kill_process(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ResponseShellKillResult]:
        """
        Terminate the process in the specified shell session

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ResponseShellKillResult]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/shell/kill",
            method="POST",
            json={
                "id": id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseShellKillResult,
                    parse_obj_as(
                        type_=ResponseShellKillResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawShellClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def exec_command(
        self,
        *,
        command: str,
        id: typing.Optional[str] = OMIT,
        exec_dir: typing.Optional[str] = OMIT,
        async_mode: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ResponseShellCommandResult]:
        """
        Execute command in the specified shell session
        Supports SSE streaming if Accept header contains 'text/event-stream'

        Parameters
        ----------
        command : str
            Shell command to execute

        id : typing.Optional[str]
            Unique identifier of the target shell session, if not provided, one will be automatically created

        exec_dir : typing.Optional[str]
            Working directory for command execution (must use absolute path)

        async_mode : typing.Optional[bool]
            Whether to execute command asynchronously (default: False for async, False for synchronous execution)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ResponseShellCommandResult]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/shell/exec",
            method="POST",
            json={
                "id": id,
                "exec_dir": exec_dir,
                "command": command,
                "async_mode": async_mode,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseShellCommandResult,
                    parse_obj_as(
                        type_=ResponseShellCommandResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def view_shell(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ResponseShellViewResult]:
        """
        View output of the specified shell session
        Supports SSE streaming if Accept header contains 'text/event-stream'

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ResponseShellViewResult]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/shell/view",
            method="POST",
            json={
                "id": id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseShellViewResult,
                    parse_obj_as(
                        type_=ResponseShellViewResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def wait_for_process(
        self, *, id: str, seconds: typing.Optional[int] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ResponseShellWaitResult]:
        """
        Wait for the process in the specified shell session to return

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        seconds : typing.Optional[int]
            Wait time (seconds)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ResponseShellWaitResult]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/shell/wait",
            method="POST",
            json={
                "id": id,
                "seconds": seconds,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseShellWaitResult,
                    parse_obj_as(
                        type_=ResponseShellWaitResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def write_to_process(
        self, *, id: str, input: str, press_enter: bool, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ResponseShellWriteResult]:
        """
        Write input to the process in the specified shell session

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        input : str
            Input content to write to the process

        press_enter : bool
            Whether to press enter key after input

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ResponseShellWriteResult]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/shell/write",
            method="POST",
            json={
                "id": id,
                "input": input,
                "press_enter": press_enter,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseShellWriteResult,
                    parse_obj_as(
                        type_=ResponseShellWriteResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def kill_process(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ResponseShellKillResult]:
        """
        Terminate the process in the specified shell session

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ResponseShellKillResult]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/shell/kill",
            method="POST",
            json={
                "id": id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseShellKillResult,
                    parse_obj_as(
                        type_=ResponseShellKillResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.response_node_js_execute_response import ResponseNodeJsExecuteResponse
from ..types.response_node_js_runtime_info import ResponseNodeJsRuntimeInfo
from .raw_client import AsyncRawNodejsClient, RawNodejsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class NodejsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawNodejsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawNodejsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawNodejsClient
        """
        return self._raw_client

    def execute_nodejs_code(
        self,
        *,
        code: str,
        timeout: typing.Optional[int] = OMIT,
        stdin: typing.Optional[str] = OMIT,
        files: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseNodeJsExecuteResponse:
        """
        Execute JavaScript code using Node.js

        This endpoint allows you to execute JavaScript code and get results back.
        Each request creates a fresh execution environment that's cleaned up automatically.

        Parameters
        ----------
        code : str
            JavaScript code to execute

        timeout : typing.Optional[int]
            Execution timeout in seconds

        stdin : typing.Optional[str]
            Standard input for the process

        files : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional files to create in execution directory

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseNodeJsExecuteResponse
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.nodejs.execute_nodejs_code(
            code="code",
        )
        """
        _response = self._raw_client.execute_nodejs_code(
            code=code, timeout=timeout, stdin=stdin, files=files, request_options=request_options
        )
        return _response.data

    def info(self, *, request_options: typing.Optional[RequestOptions] = None) -> ResponseNodeJsRuntimeInfo:
        """
        Get information about Node.js runtime and available languages

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseNodeJsRuntimeInfo
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.nodejs.info()
        """
        _response = self._raw_client.info(request_options=request_options)
        return _response.data


class AsyncNodejsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawNodejsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawNodejsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawNodejsClient
        """
        return self._raw_client

    async def execute_nodejs_code(
        self,
        *,
        code: str,
        timeout: typing.Optional[int] = OMIT,
        stdin: typing.Optional[str] = OMIT,
        files: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseNodeJsExecuteResponse:
        """
        Execute JavaScript code using Node.js

        This endpoint allows you to execute JavaScript code and get results back.
        Each request creates a fresh execution environment that's cleaned up automatically.

        Parameters
        ----------
        code : str
            JavaScript code to execute

        timeout : typing.Optional[int]
            Execution timeout in seconds

        stdin : typing.Optional[str]
            Standard input for the process

        files : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional files to create in execution directory

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseNodeJsExecuteResponse
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.nodejs.execute_nodejs_code(
                code="code",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.execute_nodejs_code(
            code=code, timeout=timeout, stdin=stdin, files=files, request_options=request_options
        )
        return _response.data

    async def info(self, *, request_options: typing.Optional[RequestOptions] = None) -> ResponseNodeJsRuntimeInfo:
        """
        Get information about Node.js runtime and available languages

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseNodeJsRuntimeInfo
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.nodejs.info()


        asyncio.run(main())
        """
        _response = await self._raw_client.info(request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .. import core
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.file_content_encoding import FileContentEncoding
from ..types.response_file_find_result import ResponseFileFindResult
from ..types.response_file_list_result import ResponseFileListResult
from ..types.response_file_read_result import ResponseFileReadResult
from ..types.response_file_replace_result import ResponseFileReplaceResult
from ..types.response_file_search_result import ResponseFileSearchResult
from ..types.response_file_upload_result import ResponseFileUploadResult
from ..types.response_file_write_result import ResponseFileWriteResult
from ..types.response_str_replace_editor_result import ResponseStrReplaceEditorResult
from .raw_client import AsyncRawFileClient, RawFileClient
from .types.command import Command

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class FileClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawFileClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawFileClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawFileClient
        """
        return self._raw_client

    def read_file(
        self,
        *,
        file: str,
        start_line: typing.Optional[int] = OMIT,
        end_line: typing.Optional[int] = OMIT,
        sudo: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseFileReadResult:
        """
        Read file content

        Parameters
        ----------
        file : str
            Absolute file path

        start_line : typing.Optional[int]
            Start line (0-based)

        end_line : typing.Optional[int]
            End line (not inclusive)

        sudo : typing.Optional[bool]
            Whether to use sudo privileges

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileReadResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.file.read_file(
            file="file",
        )
        """
        _response = self._raw_client.read_file(
            file=file, start_line=start_line, end_line=end_line, sudo=sudo, request_options=request_options
        )
        return _response.data

    def write_file(
        self,
        *,
        file: str,
        content: str,
        encoding: typing.Optional[FileContentEncoding] = OMIT,
        append: typing.Optional[bool] = OMIT,
        leading_newline: typing.Optional[bool] = OMIT,
        trailing_newline: typing.Optional[bool] = OMIT,
        sudo: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseFileWriteResult:
        """
        Write file content (supports both text and binary files)

        For binary files, set encoding to 'base64' and provide base64-encoded content.
        For text files, use default 'utf-8' encoding.

        Parameters
        ----------
        file : str
            Absolute file path

        content : str
            Content to write (text or base64 encoded for binary)

        encoding : typing.Optional[FileContentEncoding]
            Content encoding: utf-8 for text, base64 for binary data

        append : typing.Optional[bool]
            Whether to use append mode

        leading_newline : typing.Optional[bool]
            Whether to add leading newline (only for text mode)

        trailing_newline : typing.Optional[bool]
            Whether to add trailing newline (only for text mode)

        sudo : typing.Optional[bool]
            Whether to use sudo privileges

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileWriteResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.file.write_file(
            file="file",
            content="content",
        )
        """
        _response = self._raw_client.write_file(
            file=file,
            content=content,
            encoding=encoding,
            append=append,
            leading_newline=leading_newline,
            trailing_newline=trailing_newline,
            sudo=sudo,
            request_options=request_options,
        )
        return _response.data

    def replace_in_file(
        self,
        *,
        file: str,
        old_str: str,
        new_str: str,
        sudo: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseFileReplaceResult:
        """
        Replace string in file

        Parameters
        ----------
        file : str
            Absolute file path

        old_str : str
            Original string to replace

        new_str : str
            New string to replace with

        sudo : typing.Optional[bool]
            Whether to use sudo privileges

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileReplaceResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.file.replace_in_file(
            file="file",
            old_str="old_str",
            new_str="new_str",
        )
        """
        _response = self._raw_client.replace_in_file(
            file=file, old_str=old_str, new_str=new_str, sudo=sudo, request_options=request_options
        )
        return _response.data

    def search_in_file(
        self,
        *,
        file: str,
        regex: str,
        sudo: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseFileSearchResult:
        """
        Search in file content

        Parameters
        ----------
        file : str
            Absolute file path

        regex : str
            Regular expression pattern

        sudo : typing.Optional[bool]
            Whether to use sudo privileges

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileSearchResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.file.search_in_file(
            file="file",
            regex="regex",
        )
        """
        _response = self._raw_client.search_in_file(file=file, regex=regex, sudo=sudo, request_options=request_options)
        return _response.data

    def find_files(
        self, *, path: str, glob: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ResponseFileFindResult:
        """
        Find files by name pattern

        Parameters
        ----------
        path : str
            Directory path to search

        glob : str
            Filename pattern (glob syntax)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileFindResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.file.find_files(
            path="path",
            glob="glob",
        )
        """
        _response = self._raw_client.find_files(path=path, glob=glob, request_options=request_options)
        return _response.data

    def upload_file(
        self,
        *,
        file: core.File,
        path: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseFileUploadResult:
        """
        Upload file using streaming

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        path : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileUploadResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.file.upload_file()
        """
        _response = self._raw_client.upload_file(file=file, path=path, request_options=request_options)
        return _response.data

    def download_file(
        self, *, path: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Download file using FileResponse

        Parameters
        ----------
        path : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.file.download_file(
            path="path",
        )
        """
        _response = self._raw_client.download_file(path=path, request_options=request_options)
        return _response.data

    def list_path(
        self,
        *,
        path: str,
        recursive: typing.Optional[bool] = OMIT,
        show_hidden: typing.Optional[bool] = OMIT,
        file_types: typing.Optional[typing.Sequence[str]] = OMIT,
        max_depth: typing.Optional[int] = OMIT,
        include_size: typing.Optional[bool] = OMIT,
        include_permissions: typing.Optional[bool] = OMIT,
        sort_by: typing.Optional[str] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseFileListResult:
        """
        List path contents with flexible options

        Parameters
        ----------
        path : str
            Directory path to list

        recursive : typing.Optional[bool]
            Whether to list recursively

        show_hidden : typing.Optional[bool]
            Whether to show hidden files

        file_types : typing.Optional[typing.Sequence[str]]
            Filter by file extensions (e.g., ['.py', '.txt'])

        max_depth : typing.Optional[int]
            Maximum depth for recursive listing

        include_size : typing.Optional[bool]
            Whether to include file size information

        include_permissions : typing.Optional[bool]
            Whether to include file permissions

        sort_by : typing.Optional[str]
            Sort by: name, size, modified, type

        sort_desc : typing.Optional[bool]
            Sort in descending order

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileListResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.file.list_path(
            path="path",
        )
        """
        _response = self._raw_client.list_path(
            path=path,
            recursive=recursive,
            show_hidden=show_hidden,
            file_types=file_types,
            max_depth=max_depth,
            include_size=include_size,
            include_permissions=include_permissions,
            sort_by=sort_by,
            sort_desc=sort_desc,
            request_options=request_options,
        )
        return _response.data

    def str_replace_editor(
        self,
        *,
        command: Command,
        path: str,
        file_text: typing.Optional[str] = OMIT,
        old_str: typing.Optional[str] = OMIT,
        new_str: typing.Optional[str] = OMIT,
        insert_line: typing.Optional[int] = OMIT,
        view_range: typing.Optional[typing.Sequence[int]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseStrReplaceEditorResult:
        """
        An filesystem editor tool that allows the agent to
        - view
        - create
        - navigate
        - edit files
        The tool parameters are defined by Anthropic and are not editable.

        Parameters
        ----------
        command : Command
            The commands to run. Allowed options are: `view`, `create`, `str_replace`, `insert`, `undo_edit`.

        path : str
            Absolute path to file or directory, e.g. `/workspace/file.py` or `/workspace`.

        file_text : typing.Optional[str]
            Required parameter of `create` command, with the content of the file to be created.

        old_str : typing.Optional[str]
            Required parameter of `str_replace` command containing the string in `path` to replace.

        new_str : typing.Optional[str]
            Optional parameter of `str_replace` command containing the new string (if not given, no string will be added). Required parameter of `insert` command containing the string to insert.

        insert_line : typing.Optional[int]
            Required parameter of `insert` command. The `new_str` will be inserted AFTER the line `insert_line` of `path`.

        view_range : typing.Optional[typing.Sequence[int]]
            Optional parameter of `view` command when `path` points to a file. If none is given, the full file is shown. If provided, the file will be shown in the indicated line number range, e.g. [11, 12] will show lines 11 and 12. Indexing at 1 to start. Setting `[start_line, -1]` shows all lines from `start_line` to the end of the file.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseStrReplaceEditorResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.file.str_replace_editor(
            command="view",
            path="path",
        )
        """
        _response = self._raw_client.str_replace_editor(
            command=command,
            path=path,
            file_text=file_text,
            old_str=old_str,
            new_str=new_str,
            insert_line=insert_line,
            view_range=view_range,
            request_options=request_options,
        )
        return _response.data


class AsyncFileClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawFileClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawFileClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawFileClient
        """
        return self._raw_client

    async def read_file(
        self,
        *,
        file: str,
        start_line: typing.Optional[int] = OMIT,
        end_line: typing.Optional[int] = OMIT,
        sudo: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseFileReadResult:
        """
        Read file content

        Parameters
        ----------
        file : str
            Absolute file path

        start_line : typing.Optional[int]
            Start line (0-based)

        end_line : typing.Optional[int]
            End line (not inclusive)

        sudo : typing.Optional[bool]
            Whether to use sudo privileges

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileReadResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.file.read_file(
                file="file",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.read_file(
            file=file, start_line=start_line, end_line=end_line, sudo=sudo, request_options=request_options
        )
        return _response.data

    async def write_file(
        self,
        *,
        file: str,
        content: str,
        encoding: typing.Optional[FileContentEncoding] = OMIT,
        append: typing.Optional[bool] = OMIT,
        leading_newline: typing.Optional[bool] = OMIT,
        trailing_newline: typing.Optional[bool] = OMIT,
        sudo: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseFileWriteResult:
        """
        Write file content (supports both text and binary files)

        For binary files, set encoding to 'base64' and provide base64-encoded content.
        For text files, use default 'utf-8' encoding.

        Parameters
        ----------
        file : str
            Absolute file path

        content : str
            Content to write (text or base64 encoded for binary)

        encoding : typing.Optional[FileContentEncoding]
            Content encoding: utf-8 for text, base64 for binary data

        append : typing.Optional[bool]
            Whether to use append mode

        leading_newline : typing.Optional[bool]
            Whether to add leading newline (only for text mode)

        trailing_newline : typing.Optional[bool]
            Whether to add trailing newline (only for text mode)

        sudo : typing.Optional[bool]
            Whether to use sudo privileges

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileWriteResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.file.write_file(
                file="file",
                content="content",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.write_file(
            file=file,
            content=content,
            encoding=encoding,
            append=append,
            leading_newline=leading_newline,
            trailing_newline=trailing_newline,
            sudo=sudo,
            request_options=request_options,
        )
        return _response.data

    async def replace_in_file(
        self,
        *,
        file: str,
        old_str: str,
        new_str: str,
        sudo: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseFileReplaceResult:
        """
        Replace string in file

        Parameters
        ----------
        file : str
            Absolute file path

        old_str : str
            Original string to replace

        new_str : str
            New string to replace with

        sudo : typing.Optional[bool]
            Whether to use sudo privileges

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileReplaceResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.file.replace_in_file(
                file="file",
                old_str="old_str",
                new_str="new_str",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.replace_in_file(
            file=file, old_str=old_str, new_str=new_str, sudo=sudo, request_options=request_options
        )
        return _response.data

    async def search_in_file(
        self,
        *,
        file: str,
        regex: str,
        sudo: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseFileSearchResult:
        """
        Search in file content

        Parameters
        ----------
        file : str
            Absolute file path

        regex : str
            Regular expression pattern

        sudo : typing.Optional[bool]
            Whether to use sudo privileges

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileSearchResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.file.search_in_file(
                file="file",
                regex="regex",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.search_in_file(
            file=file, regex=regex, sudo=sudo, request_options=request_options
        )
        return _response.data

    async def find_files(
        self, *, path: str, glob: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ResponseFileFindResult:
        """
        Find files by name pattern

        Parameters
        ----------
        path : str
            Directory path to search

        glob : str
            Filename pattern (glob syntax)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileFindResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.file.find_files(
                path="path",
                glob="glob",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.find_files(path=path, glob=glob, request_options=request_options)
        return _response.data

    async def upload_file(
        self,
        *,
        file: core.File,
        path: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseFileUploadResult:
        """
        Upload file using streaming

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        path : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileUploadResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.file.upload_file()


        asyncio.run(main())
        """
        _response = await self._raw_client.upload_file(file=file, path=path, request_options=request_options)
        return _response.data

    async def download_file(
        self, *, path: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Download file using FileResponse

        Parameters
        ----------
        path : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.file.download_file(
                path="path",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.download_file(path=path, request_options=request_options)
        return _response.data

    async def list_path(
        self,
        *,
        path: str,
        recursive: typing.Optional[bool] = OMIT,
        show_hidden: typing.Optional[bool] = OMIT,
        file_types: typing.Optional[typing.Sequence[str]] = OMIT,
        max_depth: typing.Optional[int] = OMIT,
        include_size: typing.Optional[bool] = OMIT,
        include_permissions: typing.Optional[bool] = OMIT,
        sort_by: typing.Optional[str] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseFileListResult:
        """
        List path contents with flexible options

        Parameters
        ----------
        path : str
            Directory path to list

        recursive : typing.Optional[bool]
            Whether to list recursively

        show_hidden : typing.Optional[bool]
            Whether to show hidden files

        file_types : typing.Optional[typing.Sequence[str]]
            Filter by file extensions (e.g., ['.py', '.txt'])

        max_depth : typing.Optional[int]
            Maximum depth for recursive listing

        include_size : typing.Optional[bool]
            Whether to include file size information

        include_permissions : typing.Optional[bool]
            Whether to include file permissions

        sort_by : typing.Optional[str]
            Sort by: name, size, modified, type

        sort_desc : typing.Optional[bool]
            Sort in descending order

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseFileListResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.file.list_path(
                path="path",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_path(
            path=path,
            recursive=recursive,
            show_hidden=show_hidden,
            file_types=file_types,
            max_depth=max_depth,
            include_size=include_size,
            include_permissions=include_permissions,
            sort_by=sort_by,
            sort_desc=sort_desc,
            request_options=request_options,
        )
        return _response.data

    async def str_replace_editor(
        self,
        *,
        command: Command,
        path: str,
        file_text: typing.Optional[str] = OMIT,
        old_str: typing.Optional[str] = OMIT,
        new_str: typing.Optional[str] = OMIT,
        insert_line: typing.Optional[int] = OMIT,
        view_range: typing.Optional[typing.Sequence[int]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseStrReplaceEditorResult:
        """
        An filesystem editor tool that allows the agent to
        - view
        - create
        - navigate
        - edit files
        The tool parameters are defined by Anthropic and are not editable.

        Parameters
        ----------
        command : Command
            The commands to run. Allowed options are: `view`, `create`, `str_replace`, `insert`, `undo_edit`.

        path : str
            Absolute path to file or directory, e.g. `/workspace/file.py` or `/workspace`.

        file_text : typing.Optional[str]
            Required parameter of `create` command, with the content of the file to be created.

        old_str : typing.Optional[str]
            Required parameter of `str_replace` command containing the string in `path` to replace.

        new_str : typing.Optional[str]
            Optional parameter of `str_replace` command containing the new string (if not given, no string will be added). Required parameter of `insert` command containing the string to insert.

        insert_line : typing.Optional[int]
            Required parameter of `insert` command. The `new_str` will be inserted AFTER the line `insert_line` of `path`.

        view_range : typing.Optional[typing.Sequence[int]]
            Optional parameter of `view` command when `path` points to a file. If none is given, the full file is shown. If provided, the file will be shown in the indicated line number range, e.g. [11, 12] will show lines 11 and 12. Indexing at 1 to start. Setting `[start_line, -1]` shows all lines from `start_line` to the end of the file.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseStrReplaceEditorResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.file.str_replace_editor(
                command="view",
                path="path",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.str_replace_editor(
            command=command,
            path=path,
            file_text=file_text,
            old_str=old_str,
            new_str=new_str,
            insert_line=insert_line,
            view_range=view_range,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        ActionResponse,
        ActiveSessionsResult,
        Annotations,
        AnnotationsAudienceItem,
        AudioContent,
        BashCommandStatus,
        BlobResourceContents,
        BrowserInfoResult,
        BrowserViewport,
        Button,
        CallToolResult,
        CallToolResultContentItem,
        CallToolResultContentItem_Audio,
        CallToolResultContentItem_Image,
        CallToolResultContentItem_Resource,
        CallToolResultContentItem_ResourceLink,
        CallToolResultContentItem_Text,
        ClickAction,
        ConsoleRecord,
        DoubleClickAction,
        DragToAction,
        EmbeddedResource,
        FileContentEncoding,
        FileFindResult,
        FileInfo,
        FileListResult,
        FileReadResult,
        FileReplaceResult,
        FileSearchResult,
        FileUploadResult,
        FileWriteResult,
        HotkeyAction,
        HttpValidationError,
        ImageContent,
        JupyterExecuteResponse,
        JupyterInfoResponse,
        JupyterOutput,
        KeyDownAction,
        KeyUpAction,
        ListToolsResult,
        MouseDownAction,
        MouseUpAction,
        MoveToAction,
        NodeJsExecuteResponse,
        NodeJsOutput,
        NodeJsRuntimeInfo,
        PingBody,
        PressAction,
        Resolution,
        Resource,
        ResourceLink,
        Response,
        ResponseActiveSessionsResult,
        ResponseBrowserInfoResult,
        ResponseCallToolResult,
        ResponseFileFindResult,
        ResponseFileListResult,
        ResponseFileReadResult,
        ResponseFileReplaceResult,
        ResponseFileSearchResult,
        ResponseFileUploadResult,
        ResponseFileWriteResult,
        ResponseJupyterExecuteResponse,
        ResponseJupyterInfoResponse,
        ResponseListStr,
        ResponseListToolsResult,
        ResponseNodeJsExecuteResponse,
        ResponseNodeJsRuntimeInfo,
        ResponseShellCommandResult,
        ResponseShellKillResult,
        ResponseShellViewResult,
        ResponseShellWaitResult,
        ResponseShellWriteResult,
        ResponseStrReplaceEditorResult,
        RightClickAction,
        SandboxResponse,
        ScrollAction,
        SessionInfo,
        ShellCommandResult,
        ShellKillResult,
        ShellViewResult,
        ShellWaitResult,
        ShellWriteResult,
        StrReplaceEditorResult,
        TextContent,
        TextResourceContents,
        Tool,
        ToolAnnotations,
        TypingAction,
        ValidationError,
        ValidationErrorLocItem,
    )
    from .errors import UnprocessableEntityError
    from . import browser, file, jupyter, mcp, nodejs, sandbox, shell
    from .browser import (
        Action,
        Action_Click,
        Action_DoubleClick,
        Action_DragTo,
        Action_Hotkey,
        Action_KeyDown,
        Action_KeyUp,
        Action_MouseDown,
        Action_MouseUp,
        Action_MoveTo,
        Action_Press,
        Action_RightClick,
        Action_Scroll,
        Action_Typing,
    )
    from .client import AsyncSandbox, Sandbox
    from .file import Command
_dynamic_imports: typing.Dict[str, str] = {
    "Action": ".browser",
    "ActionResponse": ".types",
    "Action_Click": ".browser",
    "Action_DoubleClick": ".browser",
    "Action_DragTo": ".browser",
    "Action_Hotkey": ".browser",
    "Action_KeyDown": ".browser",
    "Action_KeyUp": ".browser",
    "Action_MouseDown": ".browser",
    "Action_MouseUp": ".browser",
    "Action_MoveTo": ".browser",
    "Action_Press": ".browser",
    "Action_RightClick": ".browser",
    "Action_Scroll": ".browser",
    "Action_Typing": ".browser",
    "ActiveSessionsResult": ".types",
    "Annotations": ".types",
    "AnnotationsAudienceItem": ".types",
    "AsyncSandbox": ".client",
    "AudioContent": ".types",
    "BashCommandStatus": ".types",
    "BlobResourceContents": ".types",
    "BrowserInfoResult": ".types",
    "BrowserViewport": ".types",
    "Button": ".types",
    "CallToolResult": ".types",
    "CallToolResultContentItem": ".types",
    "CallToolResultContentItem_Audio": ".types",
    "CallToolResultContentItem_Image": ".types",
    "CallToolResultContentItem_Resource": ".types",
    "CallToolResultContentItem_ResourceLink": ".types",
    "CallToolResultContentItem_Text": ".types",
    "ClickAction": ".types",
    "Command": ".file",
    "ConsoleRecord": ".types",
    "DoubleClickAction": ".types",
    "DragToAction": ".types",
    "EmbeddedResource": ".types",
    "FileContentEncoding": ".types",
    "FileFindResult": ".types",
    "FileInfo": ".types",
    "FileListResult": ".types",
    "FileReadResult": ".types",
    "FileReplaceResult": ".types",
    "FileSearchResult": ".types",
    "FileUploadResult": ".types",
    "FileWriteResult": ".types",
    "HotkeyAction": ".types",
    "HttpValidationError": ".types",
    "ImageContent": ".types",
    "JupyterExecuteResponse": ".types",
    "JupyterInfoResponse": ".types",
    "JupyterOutput": ".types",
    "KeyDownAction": ".types",
    "KeyUpAction": ".types",
    "ListToolsResult": ".types",
    "MouseDownAction": ".types",
    "MouseUpAction": ".types",
    "MoveToAction": ".types",
    "NodeJsExecuteResponse": ".types",
    "NodeJsOutput": ".types",
    "NodeJsRuntimeInfo": ".types",
    "PingBody": ".types",
    "PressAction": ".types",
    "Resolution": ".types",
    "Resource": ".types",
    "ResourceLink": ".types",
    "Response": ".types",
    "ResponseActiveSessionsResult": ".types",
    "ResponseBrowserInfoResult": ".types",
    "ResponseCallToolResult": ".types",
    "ResponseFileFindResult": ".types",
    "ResponseFileListResult": ".types",
    "ResponseFileReadResult": ".types",
    "ResponseFileReplaceResult": ".types",
    "ResponseFileSearchResult": ".types",
    "ResponseFileUploadResult": ".types",
    "ResponseFileWriteResult": ".types",
    "ResponseJupyterExecuteResponse": ".types",
    "ResponseJupyterInfoResponse": ".types",
    "ResponseListStr": ".types",
    "ResponseListToolsResult": ".types",
    "ResponseNodeJsExecuteResponse": ".types",
    "ResponseNodeJsRuntimeInfo": ".types",
    "ResponseShellCommandResult": ".types",
    "ResponseShellKillResult": ".types",
    "ResponseShellViewResult": ".types",
    "ResponseShellWaitResult": ".types",
    "ResponseShellWriteResult": ".types",
    "ResponseStrReplaceEditorResult": ".types",
    "RightClickAction": ".types",
    "Sandbox": ".client",
    "SandboxResponse": ".types",
    "ScrollAction": ".types",
    "SessionInfo": ".types",
    "ShellCommandResult": ".types",
    "ShellKillResult": ".types",
    "ShellViewResult": ".types",
    "ShellWaitResult": ".types",
    "ShellWriteResult": ".types",
    "StrReplaceEditorResult": ".types",
    "TextContent": ".types",
    "TextResourceContents": ".types",
    "Tool": ".types",
    "ToolAnnotations": ".types",
    "TypingAction": ".types",
    "UnprocessableEntityError": ".errors",
    "ValidationError": ".types",
    "ValidationErrorLocItem": ".types",
    "browser": ".",
    "file": ".",
    "jupyter": ".",
    "mcp": ".",
    "nodejs": ".",
    "providers": ".",
    "sandbox": ".",
    "shell": ".",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        # If the module_name is ".", it means we're importing a submodule
        # In that case, return the module itself, not an attribute from it
        if module_name == ".":
            return module
        # Otherwise, try to get the attribute from the module
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Action",
    "ActionResponse",
    "Action_Click",
    "Action_DoubleClick",
    "Action_DragTo",
    "Action_Hotkey",
    "Action_KeyDown",
    "Action_KeyUp",
    "Action_MouseDown",
    "Action_MouseUp",
    "Action_MoveTo",
    "Action_Press",
    "Action_RightClick",
    "Action_Scroll",
    "Action_Typing",
    "ActiveSessionsResult",
    "Annotations",
    "AnnotationsAudienceItem",
    "AsyncSandbox",
    "AudioContent",
    "BashCommandStatus",
    "BlobResourceContents",
    "BrowserInfoResult",
    "BrowserViewport",
    "Button",
    "CallToolResult",
    "CallToolResultContentItem",
    "CallToolResultContentItem_Audio",
    "CallToolResultContentItem_Image",
    "CallToolResultContentItem_Resource",
    "CallToolResultContentItem_ResourceLink",
    "CallToolResultContentItem_Text",
    "ClickAction",
    "Command",
    "ConsoleRecord",
    "DoubleClickAction",
    "DragToAction",
    "EmbeddedResource",
    "FileContentEncoding",
    "FileFindResult",
    "FileInfo",
    "FileListResult",
    "FileReadResult",
    "FileReplaceResult",
    "FileSearchResult",
    "FileUploadResult",
    "FileWriteResult",
    "HotkeyAction",
    "HttpValidationError",
    "ImageContent",
    "JupyterExecuteResponse",
    "JupyterInfoResponse",
    "JupyterOutput",
    "KeyDownAction",
    "KeyUpAction",
    "ListToolsResult",
    "MouseDownAction",
    "MouseUpAction",
    "MoveToAction",
    "NodeJsExecuteResponse",
    "NodeJsOutput",
    "NodeJsRuntimeInfo",
    "PingBody",
    "PressAction",
    "Resolution",
    "Resource",
    "ResourceLink",
    "Response",
    "ResponseActiveSessionsResult",
    "ResponseBrowserInfoResult",
    "ResponseCallToolResult",
    "ResponseFileFindResult",
    "ResponseFileListResult",
    "ResponseFileReadResult",
    "ResponseFileReplaceResult",
    "ResponseFileSearchResult",
    "ResponseFileUploadResult",
    "ResponseFileWriteResult",
    "ResponseJupyterExecuteResponse",
    "ResponseJupyterInfoResponse",
    "ResponseListStr",
    "ResponseListToolsResult",
    "ResponseNodeJsExecuteResponse",
    "ResponseNodeJsRuntimeInfo",
    "ResponseShellCommandResult",
    "ResponseShellKillResult",
    "ResponseShellViewResult",
    "ResponseShellWaitResult",
    "ResponseShellWriteResult",
    "ResponseStrReplaceEditorResult",
    "RightClickAction",
    "Sandbox",
    "SandboxResponse",
    "ScrollAction",
    "SessionInfo",
    "ShellCommandResult",
    "ShellKillResult",
    "ShellViewResult",
    "ShellWaitResult",
    "ShellWriteResult",
    "StrReplaceEditorResult",
    "TextContent",
    "TextResourceContents",
    "Tool",
    "ToolAnnotations",
    "TypingAction",
    "UnprocessableEntityError",
    "ValidationError",
    "ValidationErrorLocItem",
    "browser",
    "file",
    "jupyter",
    "mcp",
    "nodejs",
    "providers",
    "sandbox",
    "shell",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class StrReplaceEditorResult(UniversalBaseModel):
    """
    String replace editor result based on openhands_aci CLIResult
    """

    output: str = pydantic.Field()
    """
    Command execution output
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message if any
    """

    path: str = pydantic.Field()
    """
    File path that was operated on
    """

    prev_exist: bool = pydantic.Field()
    """
    Whether the file existed before operation
    """

    old_content: typing.Optional[str] = pydantic.Field(default=None)
    """
    Previous file content
    """

    new_content: typing.Optional[str] = pydantic.Field(default=None)
    """
    New file content after operation
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import contextlib
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.action_response import ActionResponse
from ..types.http_validation_error import HttpValidationError
from ..types.response_browser_info_result import ResponseBrowserInfoResult
from .types.action import Action

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBrowserClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_browser_info(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ResponseBrowserInfoResult]:
        """
        Get information about browser, like cdp url, viewport size, etc.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ResponseBrowserInfoResult]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/browser/info",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseBrowserInfoResult,
                    parse_obj_as(
                        type_=ResponseBrowserInfoResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    @contextlib.contextmanager
    def take_screenshot(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Iterator[HttpResponse[typing.Iterator[bytes]]]:
        """
        Take a screenshot of the current display.

        Returns:
            StreamingResponse: PNG image data with proper headers

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[HttpResponse[typing.Iterator[bytes]]]
            Screenshot image
        """
        with self._client_wrapper.httpx_client.stream(
            "v1/browser/screenshot",
            method="GET",
            request_options=request_options,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", None) if request_options is not None else None
                        return HttpResponse(
                            response=_response, data=(_chunk for _chunk in _response.iter_bytes(chunk_size=_chunk_size))
                        )
                    _response.read()
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()

    def execute_action(
        self, *, request: Action, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ActionResponse]:
        """
        Execute a validated action on the current display.

        Parameters
        ----------
        request : Action

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ActionResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/browser/actions",
            method="POST",
            json=convert_and_respect_annotation_metadata(object_=request, annotation=Action, direction="write"),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ActionResponse,
                    parse_obj_as(
                        type_=ActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBrowserClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_browser_info(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ResponseBrowserInfoResult]:
        """
        Get information about browser, like cdp url, viewport size, etc.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ResponseBrowserInfoResult]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/browser/info",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResponseBrowserInfoResult,
                    parse_obj_as(
                        type_=ResponseBrowserInfoResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    @contextlib.asynccontextmanager
    async def take_screenshot(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]:
        """
        Take a screenshot of the current display.

        Returns:
            StreamingResponse: PNG image data with proper headers

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]
            Screenshot image
        """
        async with self._client_wrapper.httpx_client.stream(
            "v1/browser/screenshot",
            method="GET",
            request_options=request_options,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", None) if request_options is not None else None
                        return AsyncHttpResponse(
                            response=_response,
                            data=(_chunk async for _chunk in _response.aiter_bytes(chunk_size=_chunk_size)),
                        )
                    await _response.aread()
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()

    async def execute_action(
        self, *, request: Action, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ActionResponse]:
        """
        Execute a validated action on the current display.

        Parameters
        ----------
        request : Action

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ActionResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/browser/actions",
            method="POST",
            json=convert_and_respect_annotation_metadata(object_=request, annotation=Action, direction="write"),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ActionResponse,
                    parse_obj_as(
                        type_=ActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.response_shell_command_result import ResponseShellCommandResult
from ..types.response_shell_kill_result import ResponseShellKillResult
from ..types.response_shell_view_result import ResponseShellViewResult
from ..types.response_shell_wait_result import ResponseShellWaitResult
from ..types.response_shell_write_result import ResponseShellWriteResult
from .raw_client import AsyncRawShellClient, RawShellClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ShellClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawShellClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawShellClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawShellClient
        """
        return self._raw_client

    def exec_command(
        self,
        *,
        command: str,
        id: typing.Optional[str] = OMIT,
        exec_dir: typing.Optional[str] = OMIT,
        async_mode: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseShellCommandResult:
        """
        Execute command in the specified shell session
        Supports SSE streaming if Accept header contains 'text/event-stream'

        Parameters
        ----------
        command : str
            Shell command to execute

        id : typing.Optional[str]
            Unique identifier of the target shell session, if not provided, one will be automatically created

        exec_dir : typing.Optional[str]
            Working directory for command execution (must use absolute path)

        async_mode : typing.Optional[bool]
            Whether to execute command asynchronously (default: False for async, False for synchronous execution)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseShellCommandResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.shell.exec_command(
            command="command",
        )
        """
        _response = self._raw_client.exec_command(
            command=command, id=id, exec_dir=exec_dir, async_mode=async_mode, request_options=request_options
        )
        return _response.data

    def view_shell(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ResponseShellViewResult:
        """
        View output of the specified shell session
        Supports SSE streaming if Accept header contains 'text/event-stream'

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseShellViewResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.shell.view_shell(
            id="id",
        )
        """
        _response = self._raw_client.view_shell(id=id, request_options=request_options)
        return _response.data

    def wait_for_process(
        self, *, id: str, seconds: typing.Optional[int] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> ResponseShellWaitResult:
        """
        Wait for the process in the specified shell session to return

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        seconds : typing.Optional[int]
            Wait time (seconds)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseShellWaitResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.shell.wait_for_process(
            id="id",
        )
        """
        _response = self._raw_client.wait_for_process(id=id, seconds=seconds, request_options=request_options)
        return _response.data

    def write_to_process(
        self, *, id: str, input: str, press_enter: bool, request_options: typing.Optional[RequestOptions] = None
    ) -> ResponseShellWriteResult:
        """
        Write input to the process in the specified shell session

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        input : str
            Input content to write to the process

        press_enter : bool
            Whether to press enter key after input

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseShellWriteResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.shell.write_to_process(
            id="id",
            input="input",
            press_enter=True,
        )
        """
        _response = self._raw_client.write_to_process(
            id=id, input=input, press_enter=press_enter, request_options=request_options
        )
        return _response.data

    def kill_process(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ResponseShellKillResult:
        """
        Terminate the process in the specified shell session

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseShellKillResult
            Successful Response

        Examples
        --------
        from agent_sandbox import Sandbox

        client = Sandbox(
            base_url="https://yourhost.com/path/to/api",
        )
        client.shell.kill_process(
            id="id",
        )
        """
        _response = self._raw_client.kill_process(id=id, request_options=request_options)
        return _response.data


class AsyncShellClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawShellClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawShellClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawShellClient
        """
        return self._raw_client

    async def exec_command(
        self,
        *,
        command: str,
        id: typing.Optional[str] = OMIT,
        exec_dir: typing.Optional[str] = OMIT,
        async_mode: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResponseShellCommandResult:
        """
        Execute command in the specified shell session
        Supports SSE streaming if Accept header contains 'text/event-stream'

        Parameters
        ----------
        command : str
            Shell command to execute

        id : typing.Optional[str]
            Unique identifier of the target shell session, if not provided, one will be automatically created

        exec_dir : typing.Optional[str]
            Working directory for command execution (must use absolute path)

        async_mode : typing.Optional[bool]
            Whether to execute command asynchronously (default: False for async, False for synchronous execution)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseShellCommandResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.shell.exec_command(
                command="command",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.exec_command(
            command=command, id=id, exec_dir=exec_dir, async_mode=async_mode, request_options=request_options
        )
        return _response.data

    async def view_shell(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ResponseShellViewResult:
        """
        View output of the specified shell session
        Supports SSE streaming if Accept header contains 'text/event-stream'

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseShellViewResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.shell.view_shell(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.view_shell(id=id, request_options=request_options)
        return _response.data

    async def wait_for_process(
        self, *, id: str, seconds: typing.Optional[int] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> ResponseShellWaitResult:
        """
        Wait for the process in the specified shell session to return

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        seconds : typing.Optional[int]
            Wait time (seconds)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseShellWaitResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.shell.wait_for_process(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.wait_for_process(id=id, seconds=seconds, request_options=request_options)
        return _response.data

    async def write_to_process(
        self, *, id: str, input: str, press_enter: bool, request_options: typing.Optional[RequestOptions] = None
    ) -> ResponseShellWriteResult:
        """
        Write input to the process in the specified shell session

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        input : str
            Input content to write to the process

        press_enter : bool
            Whether to press enter key after input

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseShellWriteResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.shell.write_to_process(
                id="id",
                input="input",
                press_enter=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.write_to_process(
            id=id, input=input, press_enter=press_enter, request_options=request_options
        )
        return _response.data

    async def kill_process(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ResponseShellKillResult:
        """
        Terminate the process in the specified shell session

        Parameters
        ----------
        id : str
            Unique identifier of the target shell session

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResponseShellKillResult
            Successful Response

        Examples
        --------
        import asyncio

        from agent_sandbox import AsyncSandbox

        client = AsyncSandbox(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.shell.kill_process(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.kill_process(id=id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .bash_command_status import BashCommandStatus
from .console_record import ConsoleRecord


class ShellViewResult(UniversalBaseModel):
    """
    Shell session content view result model
    """

    output: str = pydantic.Field()
    """
    Shell session output content
    """

    session_id: str = pydantic.Field()
    """
    Shell session ID
    """

    console: typing.Optional[typing.List[ConsoleRecord]] = pydantic.Field(default=None)
    """
    Console command records
    """

    status: BashCommandStatus = pydantic.Field()
    """
    Shell session status
    """

    command: typing.Optional[str] = pydantic.Field(default=None)
    """
    Last executed or currently executing command
    """

    exit_code: typing.Optional[int] = pydantic.Field(default=None)
    """
    Command execution exit code, only has value when status is completed
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .annotations import Annotations
from .resource import Resource


class CallToolResultContentItem_Text(UniversalBaseModel):
    type: typing.Literal["text"] = "text"
    text: str
    annotations: typing.Optional[Annotations] = None
    meta: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="_meta")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class CallToolResultContentItem_Image(UniversalBaseModel):
    type: typing.Literal["image"] = "image"
    data: str
    mime_type: typing_extensions.Annotated[str, FieldMetadata(alias="mimeType")]
    annotations: typing.Optional[Annotations] = None
    meta: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="_meta")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class CallToolResultContentItem_Audio(UniversalBaseModel):
    type: typing.Literal["audio"] = "audio"
    data: str
    mime_type: typing_extensions.Annotated[str, FieldMetadata(alias="mimeType")]
    annotations: typing.Optional[Annotations] = None
    meta: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="_meta")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class CallToolResultContentItem_ResourceLink(UniversalBaseModel):
    type: typing.Literal["resource_link"] = "resource_link"
    name: str
    title: typing.Optional[str] = None
    uri: str
    description: typing.Optional[str] = None
    mime_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="mimeType")] = None
    size: typing.Optional[int] = None
    annotations: typing.Optional[Annotations] = None
    meta: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="_meta")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class CallToolResultContentItem_Resource(UniversalBaseModel):
    type: typing.Literal["resource"] = "resource"
    resource: Resource
    annotations: typing.Optional[Annotations] = None
    meta: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="_meta")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


CallToolResultContentItem = typing.Union[
    CallToolResultContentItem_Text,
    CallToolResultContentItem_Image,
    CallToolResultContentItem_Audio,
    CallToolResultContentItem_ResourceLink,
    CallToolResultContentItem_Resource,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class NodeJsRuntimeInfo(UniversalBaseModel):
    """
    NodeJS runtime information model
    """

    node_version: str = pydantic.Field()
    """
    Node.js version
    """

    npm_version: str = pydantic.Field()
    """
    npm version
    """

    supported_languages: typing.List[str] = pydantic.Field()
    """
    List of supported languages
    """

    description: str = pydantic.Field()
    """
    Service description
    """

    runtime_directory: typing.Optional[str] = pydantic.Field(default=None)
    """
    Runtime directory path
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message if runtime info retrieval failed
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...types.button import Button


class Action_MoveTo(UniversalBaseModel):
    action_type: typing.Literal["MOVE_TO"] = "MOVE_TO"
    x: float
    y: float

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Action_Click(UniversalBaseModel):
    action_type: typing.Literal["CLICK"] = "CLICK"
    x: typing.Optional[float] = None
    y: typing.Optional[float] = None
    button: typing.Optional[Button] = None
    num_clicks: typing.Optional[int] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Action_MouseDown(UniversalBaseModel):
    action_type: typing.Literal["MOUSE_DOWN"] = "MOUSE_DOWN"
    button: typing.Optional[Button] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Action_MouseUp(UniversalBaseModel):
    action_type: typing.Literal["MOUSE_UP"] = "MOUSE_UP"
    button: typing.Optional[Button] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Action_RightClick(UniversalBaseModel):
    action_type: typing.Literal["RIGHT_CLICK"] = "RIGHT_CLICK"
    x: typing.Optional[float] = None
    y: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Action_DoubleClick(UniversalBaseModel):
    action_type: typing.Literal["DOUBLE_CLICK"] = "DOUBLE_CLICK"
    x: typing.Optional[float] = None
    y: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Action_DragTo(UniversalBaseModel):
    action_type: typing.Literal["DRAG_TO"] = "DRAG_TO"
    x: float
    y: float

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Action_Scroll(UniversalBaseModel):
    action_type: typing.Literal["SCROLL"] = "SCROLL"
    dx: typing.Optional[int] = None
    dy: typing.Optional[int] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Action_Typing(UniversalBaseModel):
    action_type: typing.Literal["TYPING"] = "TYPING"
    text: str
    use_clipboard: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Action_Press(UniversalBaseModel):
    action_type: typing.Literal["PRESS"] = "PRESS"
    key: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Action_KeyDown(UniversalBaseModel):
    action_type: typing.Literal["KEY_DOWN"] = "KEY_DOWN"
    key: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Action_KeyUp(UniversalBaseModel):
    action_type: typing.Literal["KEY_UP"] = "KEY_UP"
    key: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Action_Hotkey(UniversalBaseModel):
    action_type: typing.Literal["HOTKEY"] = "HOTKEY"
    keys: typing.List[str]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


Action = typing.Union[
    Action_MoveTo,
    Action_Click,
    Action_MouseDown,
    Action_MouseUp,
    Action_RightClick,
    Action_DoubleClick,
    Action_DragTo,
    Action_Scroll,
    Action_Typing,
    Action_Press,
    Action_KeyDown,
    Action_KeyUp,
    Action_Hotkey,
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .action_response import ActionResponse
    from .active_sessions_result import ActiveSessionsResult
    from .annotations import Annotations
    from .annotations_audience_item import AnnotationsAudienceItem
    from .audio_content import AudioContent
    from .bash_command_status import BashCommandStatus
    from .blob_resource_contents import BlobResourceContents
    from .browser_info_result import BrowserInfoResult
    from .browser_viewport import BrowserViewport
    from .button import Button
    from .call_tool_result import CallToolResult
    from .call_tool_result_content_item import (
        CallToolResultContentItem,
        CallToolResultContentItem_Audio,
        CallToolResultContentItem_Image,
        CallToolResultContentItem_Resource,
        CallToolResultContentItem_ResourceLink,
        CallToolResultContentItem_Text,
    )
    from .click_action import ClickAction
    from .console_record import ConsoleRecord
    from .double_click_action import DoubleClickAction
    from .drag_to_action import DragToAction
    from .embedded_resource import EmbeddedResource
    from .file_content_encoding import FileContentEncoding
    from .file_find_result import FileFindResult
    from .file_info import FileInfo
    from .file_list_result import FileListResult
    from .file_read_result import FileReadResult
    from .file_replace_result import FileReplaceResult
    from .file_search_result import FileSearchResult
    from .file_upload_result import FileUploadResult
    from .file_write_result import FileWriteResult
    from .hotkey_action import HotkeyAction
    from .http_validation_error import HttpValidationError
    from .image_content import ImageContent
    from .jupyter_execute_response import JupyterExecuteResponse
    from .jupyter_info_response import JupyterInfoResponse
    from .jupyter_output import JupyterOutput
    from .key_down_action import KeyDownAction
    from .key_up_action import KeyUpAction
    from .list_tools_result import ListToolsResult
    from .mouse_down_action import MouseDownAction
    from .mouse_up_action import MouseUpAction
    from .move_to_action import MoveToAction
    from .node_js_execute_response import NodeJsExecuteResponse
    from .node_js_output import NodeJsOutput
    from .node_js_runtime_info import NodeJsRuntimeInfo
    from .ping_body import PingBody
    from .press_action import PressAction
    from .resolution import Resolution
    from .resource import Resource
    from .resource_link import ResourceLink
    from .response import Response
    from .response_active_sessions_result import ResponseActiveSessionsResult
    from .response_browser_info_result import ResponseBrowserInfoResult
    from .response_call_tool_result import ResponseCallToolResult
    from .response_file_find_result import ResponseFileFindResult
    from .response_file_list_result import ResponseFileListResult
    from .response_file_read_result import ResponseFileReadResult
    from .response_file_replace_result import ResponseFileReplaceResult
    from .response_file_search_result import ResponseFileSearchResult
    from .response_file_upload_result import ResponseFileUploadResult
    from .response_file_write_result import ResponseFileWriteResult
    from .response_jupyter_execute_response import ResponseJupyterExecuteResponse
    from .response_jupyter_info_response import ResponseJupyterInfoResponse
    from .response_list_str import ResponseListStr
    from .response_list_tools_result import ResponseListToolsResult
    from .response_node_js_execute_response import ResponseNodeJsExecuteResponse
    from .response_node_js_runtime_info import ResponseNodeJsRuntimeInfo
    from .response_shell_command_result import ResponseShellCommandResult
    from .response_shell_kill_result import ResponseShellKillResult
    from .response_shell_view_result import ResponseShellViewResult
    from .response_shell_wait_result import ResponseShellWaitResult
    from .response_shell_write_result import ResponseShellWriteResult
    from .response_str_replace_editor_result import ResponseStrReplaceEditorResult
    from .right_click_action import RightClickAction
    from .sandbox_response import SandboxResponse
    from .scroll_action import ScrollAction
    from .session_info import SessionInfo
    from .shell_command_result import ShellCommandResult
    from .shell_kill_result import ShellKillResult
    from .shell_view_result import ShellViewResult
    from .shell_wait_result import ShellWaitResult
    from .shell_write_result import ShellWriteResult
    from .str_replace_editor_result import StrReplaceEditorResult
    from .text_content import TextContent
    from .text_resource_contents import TextResourceContents
    from .tool import Tool
    from .tool_annotations import ToolAnnotations
    from .typing_action import TypingAction
    from .validation_error import ValidationError
    from .validation_error_loc_item import ValidationErrorLocItem
_dynamic_imports: typing.Dict[str, str] = {
    "ActionResponse": ".action_response",
    "ActiveSessionsResult": ".active_sessions_result",
    "Annotations": ".annotations",
    "AnnotationsAudienceItem": ".annotations_audience_item",
    "AudioContent": ".audio_content",
    "BashCommandStatus": ".bash_command_status",
    "BlobResourceContents": ".blob_resource_contents",
    "BrowserInfoResult": ".browser_info_result",
    "BrowserViewport": ".browser_viewport",
    "Button": ".button",
    "CallToolResult": ".call_tool_result",
    "CallToolResultContentItem": ".call_tool_result_content_item",
    "CallToolResultContentItem_Audio": ".call_tool_result_content_item",
    "CallToolResultContentItem_Image": ".call_tool_result_content_item",
    "CallToolResultContentItem_Resource": ".call_tool_result_content_item",
    "CallToolResultContentItem_ResourceLink": ".call_tool_result_content_item",
    "CallToolResultContentItem_Text": ".call_tool_result_content_item",
    "ClickAction": ".click_action",
    "ConsoleRecord": ".console_record",
    "DoubleClickAction": ".double_click_action",
    "DragToAction": ".drag_to_action",
    "EmbeddedResource": ".embedded_resource",
    "FileContentEncoding": ".file_content_encoding",
    "FileFindResult": ".file_find_result",
    "FileInfo": ".file_info",
    "FileListResult": ".file_list_result",
    "FileReadResult": ".file_read_result",
    "FileReplaceResult": ".file_replace_result",
    "FileSearchResult": ".file_search_result",
    "FileUploadResult": ".file_upload_result",
    "FileWriteResult": ".file_write_result",
    "HotkeyAction": ".hotkey_action",
    "HttpValidationError": ".http_validation_error",
    "ImageContent": ".image_content",
    "JupyterExecuteResponse": ".jupyter_execute_response",
    "JupyterInfoResponse": ".jupyter_info_response",
    "JupyterOutput": ".jupyter_output",
    "KeyDownAction": ".key_down_action",
    "KeyUpAction": ".key_up_action",
    "ListToolsResult": ".list_tools_result",
    "MouseDownAction": ".mouse_down_action",
    "MouseUpAction": ".mouse_up_action",
    "MoveToAction": ".move_to_action",
    "NodeJsExecuteResponse": ".node_js_execute_response",
    "NodeJsOutput": ".node_js_output",
    "NodeJsRuntimeInfo": ".node_js_runtime_info",
    "PingBody": ".ping_body",
    "PressAction": ".press_action",
    "Resolution": ".resolution",
    "Resource": ".resource",
    "ResourceLink": ".resource_link",
    "Response": ".response",
    "ResponseActiveSessionsResult": ".response_active_sessions_result",
    "ResponseBrowserInfoResult": ".response_browser_info_result",
    "ResponseCallToolResult": ".response_call_tool_result",
    "ResponseFileFindResult": ".response_file_find_result",
    "ResponseFileListResult": ".response_file_list_result",
    "ResponseFileReadResult": ".response_file_read_result",
    "ResponseFileReplaceResult": ".response_file_replace_result",
    "ResponseFileSearchResult": ".response_file_search_result",
    "ResponseFileUploadResult": ".response_file_upload_result",
    "ResponseFileWriteResult": ".response_file_write_result",
    "ResponseJupyterExecuteResponse": ".response_jupyter_execute_response",
    "ResponseJupyterInfoResponse": ".response_jupyter_info_response",
    "ResponseListStr": ".response_list_str",
    "ResponseListToolsResult": ".response_list_tools_result",
    "ResponseNodeJsExecuteResponse": ".response_node_js_execute_response",
    "ResponseNodeJsRuntimeInfo": ".response_node_js_runtime_info",
    "ResponseShellCommandResult": ".response_shell_command_result",
    "ResponseShellKillResult": ".response_shell_kill_result",
    "ResponseShellViewResult": ".response_shell_view_result",
    "ResponseShellWaitResult": ".response_shell_wait_result",
    "ResponseShellWriteResult": ".response_shell_write_result",
    "ResponseStrReplaceEditorResult": ".response_str_replace_editor_result",
    "RightClickAction": ".right_click_action",
    "SandboxResponse": ".sandbox_response",
    "ScrollAction": ".scroll_action",
    "SessionInfo": ".session_info",
    "ShellCommandResult": ".shell_command_result",
    "ShellKillResult": ".shell_kill_result",
    "ShellViewResult": ".shell_view_result",
    "ShellWaitResult": ".shell_wait_result",
    "ShellWriteResult": ".shell_write_result",
    "StrReplaceEditorResult": ".str_replace_editor_result",
    "TextContent": ".text_content",
    "TextResourceContents": ".text_resource_contents",
    "Tool": ".tool",
    "ToolAnnotations": ".tool_annotations",
    "TypingAction": ".typing_action",
    "ValidationError": ".validation_error",
    "ValidationErrorLocItem": ".validation_error_loc_item",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ActionResponse",
    "ActiveSessionsResult",
    "Annotations",
    "AnnotationsAudienceItem",
    "AudioContent",
    "BashCommandStatus",
    "BlobResourceContents",
    "BrowserInfoResult",
    "BrowserViewport",
    "Button",
    "CallToolResult",
    "CallToolResultContentItem",
    "CallToolResultContentItem_Audio",
    "CallToolResultContentItem_Image",
    "CallToolResultContentItem_Resource",
    "CallToolResultContentItem_ResourceLink",
    "CallToolResultContentItem_Text",
    "ClickAction",
    "ConsoleRecord",
    "DoubleClickAction",
    "DragToAction",
    "EmbeddedResource",
    "FileContentEncoding",
    "FileFindResult",
    "FileInfo",
    "FileListResult",
    "FileReadResult",
    "FileReplaceResult",
    "FileSearchResult",
    "FileUploadResult",
    "FileWriteResult",
    "HotkeyAction",
    "HttpValidationError",
    "ImageContent",
    "JupyterExecuteResponse",
    "JupyterInfoResponse",
    "JupyterOutput",
    "KeyDownAction",
    "KeyUpAction",
    "ListToolsResult",
    "MouseDownAction",
    "MouseUpAction",
    "MoveToAction",
    "NodeJsExecuteResponse",
    "NodeJsOutput",
    "NodeJsRuntimeInfo",
    "PingBody",
    "PressAction",
    "Resolution",
    "Resource",
    "ResourceLink",
    "Response",
    "ResponseActiveSessionsResult",
    "ResponseBrowserInfoResult",
    "ResponseCallToolResult",
    "ResponseFileFindResult",
    "ResponseFileListResult",
    "ResponseFileReadResult",
    "ResponseFileReplaceResult",
    "ResponseFileSearchResult",
    "ResponseFileUploadResult",
    "ResponseFileWriteResult",
    "ResponseJupyterExecuteResponse",
    "ResponseJupyterInfoResponse",
    "ResponseListStr",
    "ResponseListToolsResult",
    "ResponseNodeJsExecuteResponse",
    "ResponseNodeJsRuntimeInfo",
    "ResponseShellCommandResult",
    "ResponseShellKillResult",
    "ResponseShellViewResult",
    "ResponseShellWaitResult",
    "ResponseShellWriteResult",
    "ResponseStrReplaceEditorResult",
    "RightClickAction",
    "SandboxResponse",
    "ScrollAction",
    "SessionInfo",
    "ShellCommandResult",
    "ShellKillResult",
    "ShellViewResult",
    "ShellWaitResult",
    "ShellWriteResult",
    "StrReplaceEditorResult",
    "TextContent",
    "TextResourceContents",
    "Tool",
    "ToolAnnotations",
    "TypingAction",
    "ValidationError",
    "ValidationErrorLocItem",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class TextResourceContents(UniversalBaseModel):
    """
    Text contents of a resource.
    """

    uri: str
    mime_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="mimeType")] = None
    meta: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="_meta")
    ] = None
    text: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .jupyter_output import JupyterOutput


class JupyterExecuteResponse(UniversalBaseModel):
    """
    Jupyter code execution response model
    """

    kernel_name: str = pydantic.Field()
    """
    Name of the kernel used for execution
    """

    session_id: str = pydantic.Field()
    """
    Session ID for this kernel instance
    """

    status: str = pydantic.Field()
    """
    Execution status: ok, error, or timeout
    """

    execution_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Execution count from the kernel
    """

    outputs: typing.List[JupyterOutput] = pydantic.Field()
    """
    List of execution outputs
    """

    code: str = pydantic.Field()
    """
    The executed code
    """

    msg_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Message ID from Jupyter kernel
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class FileInfo(UniversalBaseModel):
    """
    File information
    """

    name: str = pydantic.Field()
    """
    File name
    """

    path: str = pydantic.Field()
    """
    Full file path
    """

    is_directory: bool = pydantic.Field()
    """
    Whether it's a directory
    """

    size: typing.Optional[int] = pydantic.Field(default=None)
    """
    File size in bytes
    """

    modified_time: typing.Optional[str] = pydantic.Field(default=None)
    """
    Last modified time (ISO format)
    """

    permissions: typing.Optional[str] = pydantic.Field(default=None)
    """
    File permissions
    """

    extension: typing.Optional[str] = pydantic.Field(default=None)
    """
    File extension
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

BashCommandStatus = typing.Union[
    typing.Literal["running", "completed", "no_change_timeout", "hard_timeout", "terminated"], typing.Any
]
